/*
   Simpleton Sonoff Touch firmware with MQTT support
   Supports OTA update
   David Pye (C) 2016 GNU GPL v3
*/

#include <ESP8266WiFi.h>
#include <PubSubClient.h>
#include <ESP8266mDNS.h>
#include <WiFiUdp.h>
#include <ArduinoOTA.h>

#define NAME "balkon"
const char* mqtt_user = "majordomo";
const char* mqtt_password = "123456";

#define SSID "Moscow"
#define PASS "123456"

//Defaults to DHCP, if you want a static IP, uncomment and
//configure below
#define STATIC_IP
#ifdef STATIC_IP
IPAddress ip(192, 168, 0, 10);
IPAddress gateway(192, 168, 0, 1);
IPAddress subnet(255, 255, 255, 0);
#endif

#define MQTT_SERVER "192.168.0.4"
#define MQTT_PORT 1883

//#define OTA_PASS ""
//#define OTA_PORT 8266

const char *cmndTopic1 = "cmnd/" NAME "/light1";
const char *cmndTopic2 = "cmnd/" NAME "/light2";
const char *cmndTopic3 = "cmnd/" NAME "/light3";
const char *cmndTopic4 = "cmnd/group/lights";
const char *statusTopic1 = "status/" NAME "/light1";
const char *statusTopic2 = "status/" NAME "/light2";
const char *statusTopic3 = "status/" NAME "/light3";


#define BUTTON_PIN1 0
#define RELAY_PIN1 12
#define LED_PIN 13

#define BUTTON_PIN2 9
#define RELAY_PIN2 5

#define BUTTON_PIN3 10
#define RELAY_PIN3 4

volatile int desiredRelayState1 = 0;
volatile int relayState1 = 0;
volatile int desiredRelayState2 = 0;
volatile int relayState2 = 0;
volatile int desiredRelayState3 = 0;
volatile int relayState3 = 0;
volatile unsigned long millisSinceChange = 0;

unsigned long lastMQTTCheck = -5000; //This will force an immediate check on init.

WiFiClient espClient;
PubSubClient client(espClient);
bool printedWifiToSerial = false;

void initWifi() {
  // We start by connecting to a WiFi network
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(SSID);
#ifdef STATIC_IP
  WiFi.config(ip, gateway, subnet);
#endif

  WiFi.begin(SSID, PASS);
}

void checkMQTTConnection() {
  Serial.print("Checking MQTT connection: ");
  if (client.connected()) Serial.println("OK");
  else {
    if (WiFi.status() == WL_CONNECTED) {
      //Wifi connected, attempt to connect to server
      Serial.print("new connection: ");
      if (client.connect(NAME,mqtt_user,mqtt_password)) {
        Serial.println("connected");
        client.subscribe(cmndTopic1);
        client.subscribe(cmndTopic2);
        client.subscribe(cmndTopic3);
        client.subscribe(cmndTopic4);
      } else {
        Serial.print("failed, rc=");
        Serial.println(client.state());
      }
    }
    else {
      //Wifi isn't connected, so no point in trying now.
      Serial.println(" Not connected to WiFI AP, abandoned connect.");
    }
  }
  //Set the status LED to ON if we are connected to the MQTT server
  if (client.connected()) digitalWrite(LED_PIN, LOW);
    else digitalWrite(LED_PIN, HIGH);
}

void MQTTcallback(char* topic, byte* payload, unsigned int length) {
  Serial.print("Message arrived [");
  Serial.print(topic);
  Serial.println("] ");

  if (!strcmp(topic, cmndTopic1) || !strcmp(topic, cmndTopic4)) {
    if ((char)payload[0] == '1' || ! strncasecmp_P((char *)payload, "on", length)) {
        desiredRelayState1 = 1;
    }
    else if ((char)payload[0] == '0' || ! strncasecmp_P((char *)payload, "off", length)) {
      desiredRelayState1 = 0;
    }
    else if ( ! strncasecmp_P((char *)payload, "toggle", length)) {
      desiredRelayState1 = !desiredRelayState1;
    }
  }

if (!strcmp(topic, cmndTopic2) || !strcmp(topic, cmndTopic4)) {
    if ((char)payload[0] == '1' || ! strncasecmp_P((char *)payload, "on", length)) {
        desiredRelayState2 = 1;
    }
    else if ((char)payload[0] == '0' || ! strncasecmp_P((char *)payload, "off", length)) {
      desiredRelayState2 = 0;
    }
    else if ( ! strncasecmp_P((char *)payload, "toggle", length)) {
      desiredRelayState2 = !desiredRelayState2;
    }
  }

if (!strcmp(topic, cmndTopic3) || !strcmp(topic, cmndTopic4)) {
    if ((char)payload[0] == '1' || ! strncasecmp_P((char *)payload, "on", length)) {
        desiredRelayState3 = 1;
    }
    else if ((char)payload[0] == '0' || ! strncasecmp_P((char *)payload, "off", length)) {
      desiredRelayState3 = 0;
    }
    else if ( ! strncasecmp_P((char *)payload, "toggle", length)) {
      desiredRelayState3 = !desiredRelayState3;
    }
  }
}

void shortPress() {
  desiredRelayState1 = !desiredRelayState1; //Toggle relay state.
}

void longPress1() {
  desiredRelayState1 = !desiredRelayState1; //Toggle relay state.
}

void longPress2() {
  desiredRelayState2 = !desiredRelayState2; //Toggle relay state.
}

void longPress3() {
  desiredRelayState3 = !desiredRelayState3; //Toggle relay state.
}

void buttonChangeCallback1() 
  {
    if (digitalRead(0) == 0) 
      {
        longPress1();
      }
  }

void buttonChangeCallback2() 
  {
    if (digitalRead(9) == 0) 
      {
        longPress2();
      }
  }

void buttonChangeCallback3() 
  {
    if (digitalRead(10) == 0) 
      {
        longPress3();
      }
  }



  

void setup() {
  Serial.begin(115200);
  Serial.println("Initialising");
  pinMode(RELAY_PIN1, OUTPUT);
  digitalWrite(RELAY_PIN1, LOW);
  pinMode(BUTTON_PIN1, INPUT_PULLUP);

  pinMode(RELAY_PIN2, OUTPUT);
  digitalWrite(RELAY_PIN2, LOW);
  pinMode(BUTTON_PIN2, INPUT_PULLUP);

  pinMode(RELAY_PIN3, OUTPUT);
  digitalWrite(RELAY_PIN3, LOW);
  pinMode(BUTTON_PIN3, INPUT_PULLUP);
  
  pinMode(LED_PIN, OUTPUT);

  digitalWrite(LED_PIN, HIGH); //LED off.

  initWifi();

  client.setServer(MQTT_SERVER, MQTT_PORT);
  client.setCallback(MQTTcallback);

  //OTA setup
  //ArduinoOTA.setPort(OTA_PORT);
  ArduinoOTA.setHostname(NAME);
  //ArduinoOTA.setPassword(OTA_PASS);
  ArduinoOTA.begin();
  //Enable interrupt for button press

  Serial.println("Enabling touch switch interrupt");
  attachInterrupt(digitalPinToInterrupt(BUTTON_PIN1), buttonChangeCallback1, CHANGE);
  attachInterrupt(digitalPinToInterrupt(BUTTON_PIN2), buttonChangeCallback2, CHANGE);
  attachInterrupt(digitalPinToInterrupt(BUTTON_PIN3), buttonChangeCallback3, CHANGE);

  
}

void loop() {
  //If we haven't printed WiFi details to Serial port yet, and WiFi now connected,
  //do so now. (just the once)
  if (!printedWifiToSerial && WiFi.status() == WL_CONNECTED) {
    Serial.println("WiFi connected");
    Serial.println("IP address: ");
    Serial.println(WiFi.localIP());
    printedWifiToSerial = true;
  }

  if (millis() - lastMQTTCheck >= 5000) {
    checkMQTTConnection();
    lastMQTTCheck = millis();
  }

  //Handle any pending MQTT messages
  client.loop();

  //Handle any pending OTA SW updates
  ArduinoOTA.handle();

  //Relay state is updated via the interrupt *OR* the MQTT callback.
  if (relayState1 != desiredRelayState1) {
      Serial.print("Changing state to ");
      Serial.println(desiredRelayState1);

      digitalWrite(RELAY_PIN1, desiredRelayState1);
      relayState1 = desiredRelayState1;

      Serial.print("Sending MQTT status update ");
      Serial.print(relayState1);
      Serial.print(" to ");
      Serial.println(statusTopic1);

      client.publish(statusTopic1, relayState1 == 0 ? "0" : "1");
  }

if (relayState2 != desiredRelayState2) {
      Serial.print("Changing state to ");
      Serial.println(desiredRelayState2);

      digitalWrite(RELAY_PIN2, desiredRelayState2);
      relayState2 = desiredRelayState2;

      Serial.print("Sending MQTT status update ");
      Serial.print(relayState2);
      Serial.print(" to ");
      Serial.println(statusTopic2);

      client.publish(statusTopic2, relayState2 == 0 ? "0" : "1");
  }

if (relayState3 != desiredRelayState3) {
      Serial.print("Changing state to ");
      Serial.println(desiredRelayState3);

      digitalWrite(RELAY_PIN3, desiredRelayState3);
      relayState3 = desiredRelayState3;

      Serial.print("Sending MQTT status update ");
      Serial.print(relayState3);
      Serial.print(" to ");
      Serial.println(statusTopic3);

      client.publish(statusTopic3, relayState3 == 0 ? "0" : "1");
  }


  
  delay(50);
}
